"""mod_python interface module

Move along, nothing to see here.

"""

import os
import os.path
import stat
import sys
import logging
try:
    from mod_python import apache
    from mod_python import util
    from mod_python import Cookie
except ImportError:
    print "You need to use this module from mod_python"
    pass
import WebFrame as WF
from WebFrame import reqresp
from WebFrame import handler
from WebFrame import auth

__initdone = False

class Request(reqresp.Request):
    """The mod_perl enabled request object"""
    def __init__(self, req):
        reqresp.Request.__init__(self)
        self._req = req
        self.uri = req.uri
        self.method = req.method
        self.content_length = req.clength
        self.query_string = req.args
        self.filename = req.filename
        self.docroot = req.document_root().rstrip("/")
        self.cookies = Cookie.get_cookies(self._req)
        self.__fs = None

        # Process path info
        if req.path_info != None and len(req.path_info) == 0:
            self.path_info = None
        else:
            self.path_info = req.path_info

        # Get content type
        if "Content-Type" in req.headers_in:
            self.content_type = req.headers_in["Content-Type"]
        else:
            self.content_type = "text/plain"


        # Check if we need to read the body, or if it is consumed by
        # the post data decoding
        try:
            self.__fs = util.FieldStorage(req)
        except Exception, e:
            self._body = req.read()

        # Parse post/get parameters
        self._parse_query()

        self.resp = Response(self)

    def _parse_query(self):
        if self.__fs:
            # If fieldstorage parsed the URL + POST-data
            for key in self.__fs.keys():
                if key in self.params:
                    self.params[key] += self.__fs.getlist(key)
                else:
                    self.params[key] = self.__fs.getlist(key)
        else:
            reqresp.Request._parse_query(self)

class Response(reqresp.Response):
    def __init__(self, request):
        """Do not create this object. The system  does that"""
        reqresp.Response.__init__(self, request)
        self._req = request._req
        self._req.content_type = self.content_type

    def _do_set_ct(self, ct):
        self._req.content_type = ct

    def _do_write(self, data):
        self._req.write(data)

    def status(self, status):
        self._req.status = status

    def set_cookie(self, name, value, timestamp):
        """Sets a cookie.
        You must call this before calling any of the write methods
        since those aren't currently buffered"""
        c = Cookie.Cookie(name, value)
        c.expires = timestamp
        Cookie.add_cookie(self._req, c)
        return True

    def set_header(self, name, value):
        self._req.headers_out[name] = value
        return True

class ModPythonLogger(logging.Handler):
    def __init__(self, req):
        logging.Handler.__init__(self)
        self.req = req
    def emit(self, msg):
        self.req.log_error(self.format(msg))

def _init(req):
    # Global system init, if not done already
    global __initdone
    if not __initdone:
        WF._init(req.get_options()["APPLICATION"],
                logger=ModPythonLogger(req))
        __initdone = True    
        if WF.get_opt("debug"):
            logging.debug("Init complete")

def make_ctx(req):
    if req.filename and req.path_info:
        # Magically try to clean up path-info
        prefix = req.filename[len(req.document_root()):]
        if req.path_info.startswith(prefix):
            req.path_info = req.path_info[len(prefix):]
        else:
            # Strip ending
            prefix, trash = prefix.rsplit(".", 1)
            if req.path_info.startswith(prefix):
                req.path_info = req.path_info[len(prefix):]
        req.path_info = req.path_info.strip("/")
            
    r =  Request(req)
    ctx = WF.Context(r, r.resp)

    # Check basic auth user
    pwd = req.get_basic_auth_pw()
    user = req.user
    if pwd != None and user != None:
        u = auth.get_user(user, pwd)
        ctx._set_user(u)
    return ctx
    
def template_handler(req):
    if WF.get_opt("debug"):
        logging.debug("Template handler got request for '%s'", req.filename)
    _init(req)
    ctx = make_ctx(req)
    try:
        WF._process_template(ctx)
    except reqresp.StopProcessing:
        pass
    return apache.DONE

def wf_handler(req):
    if WF.get_opt("debug"):
        logging.debug("WF handler got request for '%s'", req.filename)
    _init(req)
    ctx = make_ctx(req)
    try:
        WF._process_wf(ctx)
    except reqresp.StopProcessing:
        pass
    return apache.DONE

def handler_handler(req):
    if WF.get_opt("debug"):
        logging.debug("Handler handler got request for '%s'", req.filename)
    _init(req)
    ctx = make_ctx(req)
    try:
        WF._process_handler(ctx, req.webframe_handler)
    except reqresp.StopProcessing:
        pass
    return apache.DONE

def dir_handler(req):
    if WF.get_opt("debug"):
        logging.debug("Dir handler got request for '%s'", req.filename)
    _init(req)

    # Find index file, wf first then html
    if os.access(req.filename+"/index.wf", os.R_OK):
        req.filename += "/index.wf"
        pf = WF._process_wf
    elif os.access(req.filename+"/index.html", os.R_OK):
        req.filename += "/index.html"
        pf = WF._process_template
    else:
        # No index, too bad
        return apache.DECLINED
    
    ctx = make_ctx(req)
    try:
        pf(ctx)
    except reqresp.StopProcessing:
        pass
    return apache.DONE

def apache_handler(req):
    if WF.get_opt("debug"):
        logging.debug("Letting apache deal with '%s'", req.uri)
    return apache.DECLINED

def check_path(path):
    if WF.get_opt("debug"):
        logging.debug("Checking '%s'", path)
    # check for .wf
    if os.access(path+".wf", os.R_OK):
        return path+".wf", "wf"
    elif not WF.get_opt("html.allow_pathinfo"):
        return None, None
    elif os.access(path+".html", os.R_OK):
        return path+".html", "template"
    else:
        return None, None

def translate(req):
    _init(req)

    docroot = req.document_root().rstrip("/")
    uri = req.uri.rstrip("/")
    
    # Transform uri to real path
    fname = docroot+uri
    fname = os.path.realpath(fname)

    # Check that within docroot
    fname2 = fname+"/"
    if not fname2.startswith(docroot+"/"):
        logging.error("URI: '%s' translates to '%s' which isn't in docroot",
                      req.uri, fname)
        return apache.DECLINED

    # We check if the uri is an absolute reference to a file or directory
    # in the webspace. If it is, we don't do path finding but just dispatch
    if os.access(fname, os.R_OK):
        st = os.stat(fname)
        if stat.S_ISDIR(st.st_mode):
            # If uri points to an actual directory, let dirhandler do it
            if WF.get_opt("debug"):
                    logging.debug("Set dir handler for '%s'", uri)
            req.add_handler("PythonHandler", "WebFrame.mod::dir_handler")
            req.filename = fname
            return apache.OK
        else:
            # If uri points to actual file, let apache deal with it
            if WF.get_opt("debug"):
                logging.debug("translate: '%s' points to file, delegate to"
                              +" apache", uri)
            return apache.DECLINED


    # Check if a handler is registered for this
    h, suff = handler.find(req.method, uri)
    if h:
        if WF.get_opt("debug"):
            logging.debug("Found handler for '%s'", uri)
        req.webframe_handler = h
        req.path_info = suff
        req.add_handler("PythonHandler", "WebFrame.mod::handler_handler")
        return apache.OK
    
    # Try to find .wf file or template
    path_info = ""
    fname = fname[len(docroot):]
    while len(fname) > 0:
        f, t = check_path(docroot+fname)
        if f:
            req.filename = f
            req.path_info = path_info
            if t == "wf":
                if WF.get_opt("debug"):
                    logging.debug("Set WF handler for '%s'", uri)
                req.add_handler("PythonHandler", "WebFrame.mod::wf_handler")
            elif t == "template":
                if WF.get_opt("debug"):
                    logging.debug("Set Template handler for '%s'", uri)
                req.add_handler("PythonHandler",
                                "WebFrame.mod::template_handler")
            return apache.OK
        else:
            fname, suffix = fname.rsplit("/", 1)
            path_info = suffix+"/"+path_info

    # No such luck
    if WF.get_opt("debug"):
        logging.debug("translate: No translation found for '%s'", uri)
    return apache.DECLINED
    
